@@ -146,8 +146,10 @@ class TreeNode(object):
     def _get_children(self):
         return self._children
     def _set_children(self, value):
-        if type(value) == list and \
-           len(set([type(n)==type(self) for n in value]))<2:
+        if type(value) == list:
+            for n in value:
+                if type(n) != type(self):
+                    raise TreeError("Incorrect child node type")
             self._children = value
         else:
             raise TreeError("Incorrect children type")
@@ -831,7 +833,7 @@ class TreeNode(object):
 
         ::
 
-             t.get_newick(features=["species","name"], format=1)
+             t.write(features=["species","name"], format=1)
 
         """
 
@@ -884,10 +886,19 @@ class TreeNode(object):
         # Convert node names into node instances
         target_nodes = _translate_nodes(self, *target_nodes)
 
-        # If only one node is provided, use self as the second target
-        if type(target_nodes) != list:
-            target_nodes = [target_nodes, self]
 
+        if type(target_nodes) != list:
+            # If only one node is provided and is the same as the seed node,
+            # return itself
+            if target_nodes is self:
+                if get_path:
+                    return self, {}
+                else:
+                    return self
+            else:
+                #Otherwise find the common ancestor of current seed node and
+                #the target_node provided
+                target_nodes = [target_nodes, self]
 
         n2path = {}
         reference = []
@@ -1147,6 +1158,11 @@ class TreeNode(object):
                 break
             else:
                 current = current.up
+
+        # if we reached the root, the tree is already at midpoint. Return any child as valid outgroup
+        if current is None:
+            current = self.children[0]
+
         return current
 
     def populate(self, size, names_library=None, reuse_names=False,
@@ -1309,17 +1325,30 @@ class TreeNode(object):
         outgroup2.dist = middist
         outgroup2.support = outgroup.support
 
-    def unroot(self):
+    def unroot(self, mode='legacy'):
         """
         Unroots current node. This function is expected to be used on
         the absolute tree root node, but it can be also be applied to
         any other internal node. It will convert a split into a
         multifurcation.
+
+        :argument "legacy" mode: The value can be "legacy" or "keep".
+        If value is "keep", then function keeps the distance between
+        the leaves by adding the distance associated to the deleted
+        edge to the remaining edge. In the other case the distance
+        value of the deleted edge is dropped
+
         """
+        if not (mode == 'legacy' or mode == 'keep'):
+            raise ValueError("The value of the mode parameter must be 'legacy' or 'keep'")
         if len(self.children)==2:
             if not self.children[0].is_leaf():
+                if mode == "keep":
+                    self.children[1].dist+=self.children[0].dist
                 self.children[0].delete()
             elif not self.children[1].is_leaf():
+                if mode == "keep":
+                    self.children[0].dist+=self.children[1].dist
                 self.children[1].delete()
             else:
                 raise TreeError("Cannot unroot a tree with only two leaves")
@@ -1350,7 +1379,7 @@ class TreeNode(object):
         :var px units: "px": pixels, "mm": millimeters, "in": inches
         :var None h: height of the image in :attr:`units`
         :var None w: width of the image in :attr:`units`
-        :var 300 dpi: dots per inches.
+        :var 90 dpi: dots per inches.
 
         """
 
@@ -1538,13 +1567,8 @@ class TreeNode(object):
         .. versionadded: 2.1
 
         Sort the branches of a given tree by node names. After the
-        tree is sorted, nodes are labeled in ascending order. This
-        can be used to ensure that nodes in a tree with the same node
-        names are always labeled in the same way. Note that if
-        duplicated names are present, extra criteria should be added
-        to sort nodes.
-
-        Unique id is stored as a node._nid attribute
+        tree is sorted. Note that if duplicated names are present,
+        extra criteria should be added to sort nodes.
 
         """
 
@@ -2333,6 +2357,107 @@ class TreeNode(object):
         for n in target:
             _resolve(n)
 
+    def cophenetic_matrix(self):
+        """
+        .. versionadded: 3.1.1
+
+        Generate a cophenetic distance matrix of the treee to standard output
+
+        The `cophenetic matrix <https://en.wikipedia.org/wiki/Cophenetic>` is a matrix representation of the
+        distance between each node.
+
+        if we have a tree like
+
+                                   ----A
+                      _____________|y
+                      |            |
+                      |            ----B
+              ________|z
+                      |            ----C
+                      |            |
+                      |____________|x     -----D
+                                   |      |
+                                   |______|w
+                                          |
+                                          |
+                                          -----E
+
+        Where w,x,y,z are internal nodes.
+        d(A,B) = d(y,A) + d(y,B)
+        and
+        d(A, E) = d(z,A) + d(z, E) = {d(z,y) + d(y,A)} + {d(z,x) + d(x,w) + d(w,E)}
+
+        We use an idea inspired by the ete3 team: https://gist.github.com/jhcepas/279f9009f46bf675e3a890c19191158b :
+
+        For each node find its path to the root.
+
+        e.g.
+
+        A -> A, y, z
+        E -> E, w, x,z
+
+        and make these orderless sets. Then we XOR the two sets to only find the elements
+        that are in one or other sets but not both. In this case A, E, y, x, w.
+
+        The distance between the two nodes is the sum of the distances from each of those nodes
+        to the parent
+
+        One more optimization: since the distances are symmetric, and distance to itself is zero
+        we user itertools.combinations rather than itertools.permutations. This cuts our computes from theta(n^2)
+        1/2n^2 - n (= O(n^2), which is still not great, but in reality speeds things up for large trees).
+
+
+        For this tree, we will return the two dimensional array:
+                         A                  B                   C                   D                     E
+        A                0           d(A-y) + d(B-y)     d(A-z) + d(C-z)     d(A-z) + d(D-z)       d(A-z) + d(E-z)
+        B         d(B-y) + d(A-y)           0            d(B-z) + d(C-z)     d(B-z) + d(D-z)       d(B-z) + d(E-z)
+        C         d(C-z) + d(A-z)    d(C-z) + d(B-z)            0            d(C-x) + d(D-x)       d(C-x) + d(E-x)
+        D         d(D-z) + d(A-z)    d(D-z) + d(B-z)     d(D-x) + d(C-x)            0              d(D-w) + d(E-w)
+        E         d(E-z) + d(A-z)    d(E-z) + d(B-z)     d(E-x) + d(C-x)     d(E-w) + d(D-w)              0
+
+        We will also return the one dimensional array with the leaves in the order in which they appear in the matrix
+        (i.e. the column and/or row headers).
+
+        :param filename: the optional file to write to. If not provided, output will be to standard output
+        :return: two-dimensional array and a one dimensional array
+        """
+
+        leaves = self.get_leaves()
+        paths = {x: set() for x in leaves}
+
+        # get the paths going up the tree
+        # we get all the nodes up to the last one and store them in a set
+
+        for n in leaves:
+            if n.is_root():
+                continue
+            movingnode = n
+            while not movingnode.is_root():
+                paths[n].add(movingnode)
+                movingnode = movingnode.up
+
+        # now we want to get all pairs of nodes using itertools combinations. We need AB AC etc but don't need BA CA
+
+        leaf_distances = {x.name: {} for x in leaves}
+
+        for (leaf1, leaf2) in itertools.combinations(leaves, 2):
+            # figure out the unique nodes in the path
+            uniquenodes = paths[leaf1] ^ paths[leaf2]
+            distance = sum(x.dist for x in uniquenodes)
+            leaf_distances[leaf1.name][leaf2.name] = leaf_distances[leaf2.name][leaf1.name] = distance
+
+        allleaves = sorted(leaf_distances.keys()) # the leaves in order that we will return
+
+        output = [] # the two dimensional array that we will return
+
+        for i, n in enumerate(allleaves):
+            output.append([])
+            for m in allleaves:
+                if m == n:
+                    output[i].append(0) # distance to ourself = 0
+                else:
+                    output[i].append(leaf_distances[n][m])
+        return output, allleaves
 
     def add_face(self, face, column, position="branch-right"):
         """
@@ -2376,14 +2501,14 @@ class TreeNode(object):
 
     @staticmethod
     def from_parent_child_table(parent_child_table):
-        """Converts a parent-child table into an ETE Tree instance. 
-        
+        """Converts a parent-child table into an ETE Tree instance.
+
         :argument parent_child_table: a list of tuples containing parent-child
            relationships. For example: [("A", "B", 0.1), ("A", "C", 0.2), ("C",
            "D", 1), ("C", "E", 1.5)]. Where each tuple represents: [parent, child,
            child-parent-dist]
-        
-        :returns: A new Tree instance 
+
+        :returns: A new Tree instance
 
         :example:
 
@@ -2418,20 +2543,20 @@ class TreeNode(object):
 
     @staticmethod
     def from_skbio(skbio_tree, map_attributes=None):
-        """Converts a scikit-bio TreeNode object into ETE Tree object. 
-        
-        :argument skbio_tree: a scikit bio TreeNode instance 
+        """Converts a scikit-bio TreeNode object into ETE Tree object.
+
+        :argument skbio_tree: a scikit bio TreeNode instance
 
         :argument None map_attributes: A list of attribute nanes in the
            scikit-bio tree that should be mapped into the ETE tree
            instance. (name, id and branch length are always mapped)
 
-        :returns: A new Tree instance 
+        :returns: A new Tree instance
 
         :example:
 
         >>> tree = Tree.from_skibio(skbioTree, map_attributes=["value"])
-        
+
         """
         from skbio import TreeNode as skbioTreeNode
 
@@ -2461,15 +2586,15 @@ class TreeNode(object):
         from .. import _ph
         _ph.call()
 
-
 def _translate_nodes(root, *nodes):
     name2node = dict([ [n, None] for n in nodes if type(n) is str])
-    for n in root.traverse():
-        if n.name in name2node:
-            if name2node[n.name] is not None:
-                raise TreeError("Ambiguous node name: "+str(n.name))
-            else:
-                name2node[n.name] = n
+    if name2node:
+        for n in root.traverse():
+            if n.name in name2node:
+                if name2node[n.name] is not None:
+                    raise TreeError("Ambiguous node name: "+str(n.name))
+                else:
+                    name2node[n.name] = n
 
     if None in list(name2node.values()):
         notfound = [key for key, value in six.iteritems(name2node) if value is None]
@@ -2492,8 +2617,3 @@ def _translate_nodes(root, *nodes):
 # Alias
 #: .. currentmodule:: ete3
 Tree = TreeNode
-
-
-
-
-
